const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/AddonManager.jsm");

/* Global variables */
var gWindow = null;
var gMenuId = null;
var gPrefs = null;
var gStringBundle = null;

let ImageBlocker = {
	classDescription: "Mobile Image Blocker",
	classID: Components.ID("{5bdea032-22fb-4a63-a753-8a92401b77b1}"),
	contractID: "@martynhaigh/blockimages-policy;1",
	xpcom_categories: ["content-policy"],
	enabled: true,

	init: function(aEnabled) {
		let registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
		registrar.registerFactory(this.classID, this.classDescription, this.contractID, this);

		let catMan = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
		for each (let category in this.xpcom_categories) {
			catMan.addCategoryEntry(category, this.contractID, this.contractID, false, true);
		}
		this.enabled = aEnabled;
	},

	uninit: function() {
		for each(let category in this.xpcom_categories) {
			catMan.deleteCategoryEntry(category, this.contractID, false);
		}

		// This needs to run asynchronously, see bug 753687
		Services.tm.currentThread.dispatch(function() {
			registrar.unregisterFactory(this.classID, this);
		}.bind(this), Ci.nsIEventTarget.DISPATCH_NORMAL);
	},

	setEnabled: function(aEnabled) {

		this.enabled = aEnabled;
	},

	toggleCheckbox: function(aCheckbox, aWindow) {
		this.setEnabled(!this.enabled);

		gPrefs.setBoolPref("enabled", this.enabled);
		aWindow.menu.update(gMenuId, { checked: this.enabled });
	},
	
	// nsIContentPolicy interface implementation
	shouldLoad: function(contentType, contentLocation, requestOrigin, node, mimeTypeGuess, extra) {
		/*dump("MH shouldLoad: " + contentType + " " +
													(contentLocation ? contentLocation.spec : "null") + " " +
													(requestOrigin ? requestOrigin.spec : "null") + " " +
													node + " " +
													mimeTypeGuess + "\n");
		*/
		if (this.enabled && contentType === Ci.nsIContentPolicy.TYPE_IMAGE) {
		    return Ci.nsIContentPolicy.REJECT;
		  
		}
		return Ci.nsIContentPolicy.ACCEPT;
	},

	shouldProcess: function(contentType, contentLocation, requestOrigin, node, mimeTypeGuess, extra) {
		return Ci.nsIContentPolicy.ACCEPT;
	},

	// nsIFactory interface implementation
	createInstance: function(outer, iid) {
		if (outer)
			throw Cr.NS_ERROR_NO_AGGREGATION;
		return this.QueryInterface(iid);
	},

	// nsISupports interface implementation
	QueryInterface: XPCOMUtils.generateQI([Ci.nsIContentPolicy, Ci.nsIFactory])


};

function isNativeUI() {
  return (Services.appinfo.ID == "{aa3c5121-dab2-40e2-81ca-7ea25febc110}");
}

function dump(a) {
  Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService).logStringMessage(a);
}

function install(params, reason) {
	gPrefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("martynhaigh.imageblocker.");
	//gf gPrefs.setBoolPref("enabled", true);
}

function startup(params, reason) {
	gPrefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("martynhaigh.imageblocker.");

	gAddonData = params;
	processWindows();

	Services.wm.addListener(windowListener);
}

function processWindows() {
	let windows = Services.wm.getEnumerator("navigator:browser");
	while (windows.hasMoreElements()) {
		let domWindow = windows.getNext().QueryInterface(Ci.nsIDOMWindow);
		loadIntoWindow(domWindow);
	}
}

function loadIntoWindow(aWindow) {
	if (!aWindow || !isNativeUI()) return;

	if (!gStringBundle)
		gStringBundle = Services.strings.createBundle("chrome://imageblock/locale/imageblock.properties");

	// Have a reference to the native window.
	gWindow = aWindow.NativeWindow;

	let menuString = gStringBundle.GetStringFromName("imageblock.menu");
	
	gMenuId =  gWindow.menu.add({
		name: menuString,
		icon: gAddonData.resourceURI.spec + "icon.png", 
		callback: function() {
			ImageBlocker.toggleCheckbox(this, gWindow);
		},
		checkable: true
	});

	let enabled = gPrefs.getBoolPref("enabled");
	if(enabled) {
		gWindow.menu.update(gMenuId, { checked: enabled });
	}
	ImageBlocker.init(enabled);
}

var windowListener = {
	onOpenWindow: function(aWindow) {
		// Wait for the window to finish loading
		let domWindow = aWindow.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindowInternal || Ci.nsIDOMWindow);
		domWindow.addEventListener("load", function() {
			domWindow.removeEventListener("load", arguments.callee, false);
			loadIntoWindow(domWindow);
		}, false);
	},

	onCloseWindow: function(aWindow) {},

	onWindowTitleChange: function(aWindow, aTitle) {}
}

function unloadFromWindow(aWindow) {
	dump("MH - unload");
  if (!aWindow || !isNativeUI())
    return;

  // Remove the menu item.
  aWindow.NativeWindow.menu.remove(gMenuId);

  // Uninitialize
  ImageBlocker.uninit();
}


function shutdown(aData, aReason) {
  // When the application is shutting down we normally don't have to clean
  // up any UI changes made
  if (aReason == APP_SHUTDOWN)
    return;

  // Stop listening for new windows
  Services.wm.removeListener(windowListener);

  // Unload from any existing windows
  let windows = Services.wm.getEnumerator("navigator:browser");
  while (windows.hasMoreElements()) {
    let domWindow = windows.getNext().QueryInterface(Ci.nsIDOMWindow);
    unloadFromWindow(domWindow);
  }


}


function uninstall(params, reason) {
}